#include "GLWindow.h"
#include <iostream>
#include <ngl/Vector.h>
#include <ngl/Light.h>
#include <ngl/NGLInit.h>
#include <ngl/VBOPrimitives.h>
#include <ngl/Material.h>
#include <ngl/ShaderManager.h>
#include <QColorDialog>


//----------------------------------------------------------------------------------------------------------------------
GLWindow::GLWindow(
                   QWidget *_parent
                  ) :
                    QGLWidget(_parent)
{

  // set this widget to have the initial keyboard focus
  setFocus();
  // re-size the widget to that of the parent (in this case the GLFrame passed in on construction)
  this->resize(_parent->size());
	m_wireframe=false;
	m_rotation=0.0;
	m_scale=1.0;
	m_position=0.0;

	m_selectedObject=0;

}

// This virtual function is called once before the first call to paintGL() or resizeGL(),
//and then once whenever the widget has been assigned a new QGLContext.
// This function should set up any required OpenGL context rendering flags, defining display lists, etc.

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::initializeGL()
{

  glClearColor(0.4f, 0.4f, 0.4f, 1.0f);			   // Grey Background
  // enable depth testing for drawing
  glEnable(GL_DEPTH_TEST);
	std::cout<<"ngl init\n";
	ngl::NGLInit *init = ngl::NGLInit::instance();
	#ifdef WIN32
		glewInit();
	#endif
	init->initGlew();
	init->initVBO();
	/// create our camera
	ngl::Vector eye(2,2,2);
	ngl::Vector look(0,0,0);
	ngl::Vector up(0,1,0);

	m_camera = new ngl::Camera(eye,look,up,ngl::PERSPECTIVE);
	m_camera->setShape(45,float(1024/720),0.1,300,ngl::PERSPECTIVE);
	std::cout<<"creating shader man\n";
	ngl::ShaderManager *shader=ngl::ShaderManager::instance();
	std::cout<<"done\n";
	// create shader program
	shader->createShaderProgram("Blinn");
	// load shader sources
	shader->attachShader("vertex",ngl::VERTEX);
	shader->attachShader("fragment",ngl::FRAGMENT);
	shader->loadShaderSource("vertex","shaders/Vertex.vs");
	shader->loadShaderSource("fragment","shaders/Fragment.fs");
	// compile
	shader->compileShader("vertex");
	shader->compileShader("fragment");
	// link
	shader->attachShaderToProgram("Blinn","vertex");
	shader->attachShaderToProgram("Blinn","fragment");
	shader->linkProgramObject("Blinn");
	// use
	(*shader)["Blinn"]->use();

	ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();
	prim->createVBOSphere("sphere",1.0,40);
	ngl::Light *l = new ngl::Light(ngl::Vector(0,1,1),ngl::Colour(1,1,1),ngl::LIGHTLOCAL);
	l->enable();
	ngl::Material m(ngl::BLACKPLASTIC);
	m.use();
}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget has been resized.
// The new size is passed in width and height.
void GLWindow::resizeGL(
                        int _w,
                        int _h
                       )
{
  glViewport(0,0,_w,_h);
	m_camera->setShape(45,float(_w/_h),0.1,300,ngl::PERSPECTIVE);
	ngl::ShaderManager *shader=ngl::ShaderManager::instance();
	(*shader)["Blinn"]->use();
	shader->setShaderParamFromMatrix("Blinn","projectionMatrix",m_camera->getProjection());
}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget needs to be painted.
// this is our main drawing routine
void GLWindow::paintGL()
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();
	if(m_wireframe == true)
	{
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	}
	else
	{
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	}
	ngl::ShaderManager *shader=ngl::ShaderManager::instance();
	(*shader)["Blinn"]->use();

	m_transformStack.pushTransform();
	{
		m_transformStack.setPosition(m_position);
		m_transformStack.setScale(m_scale);
		m_transformStack.setRotation(m_rotation);
		shader->setShaderParamFromMatrix("Blinn","ViewMatrix",m_camera->getModelView());
		m_transformStack.loadGlobalAndCurrentMatrixToShader("Blinn","ModelMatrix");
		switch(m_selectedObject)
		{
			case 0 : prim->draw("teapot"); break;
			case 1 : prim->draw("sphere"); break;
			case 2 : prim->draw("cube"); break;
		}
	}
	m_transformStack.popTransform();
}





//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mouseMoveEvent (
                               QMouseEvent * _event
                              )
{
}


//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mousePressEvent (
                                QMouseEvent * _event
                               )
{

}

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mouseReleaseEvent (
                                  QMouseEvent * _event
                                 )
{
}


GLWindow::~GLWindow()
{
}

void GLWindow::toggleWireframe(
															 bool _mode
															 )
{
	m_wireframe=_mode;
	updateGL();
}

void GLWindow::setXRotation(
														 double _x
											   		)
{
	m_rotation.m_x=_x;
	updateGL();
}

void GLWindow::setYRotation(
														 double _y
														)
{
	m_rotation.m_y=_y;
	updateGL();
}
void GLWindow::setZRotation(
														 double _z
													 )
{
	m_rotation.m_z=_z;
	updateGL();
}

void GLWindow::setXScale(
														 double _x
											   		)
{
	m_scale.m_x=_x;
	updateGL();
}

void GLWindow::setYScale(
														 double _y
														)
{
	m_scale.m_y=_y;
	updateGL();
}
void GLWindow::setZScale(
														 double _z
													 )
{
	m_scale.m_z=_z;
	updateGL();
}

void GLWindow::setXPosition(
														 double _x
											   		)
{
	m_position.m_x=_x;
	updateGL();
}

void GLWindow::setYPosition(
														 double _y
														)
{
	m_position.m_y=_y;
	updateGL();
}
void GLWindow::setZPosition(
														 double _z
													 )
{
	m_position.m_z=_z;
	updateGL();
}

void GLWindow::setObjectMode(
															int _i
														)
{
	m_selectedObject=_i;
	updateGL();
}
void GLWindow::setColour()
{
	QColor colour = QColorDialog::getColor();
	if( colour.isValid())
	{
		ngl::ShaderManager *shader=ngl::ShaderManager::instance();
		(*shader)["Blinn"]->use();
		shader->setShaderParam4f("Blinn","Colour",colour.redF(),colour.greenF(),colour.blueF(),1.0);
		updateGL();
	}
}
